@@ -3,14 +3,15 @@ import numbers
 from typing import Any, Dict, List, Optional, Tuple, Union
 from warnings import warn
 
+import affine
 import geopandas as gpd
 import numpy as np
+import orjson as json
 import pyproj
 import rasterio as rio
 import rasterio.features as rio_features
 import rasterio.mask as rio_mask
 import rasterio.transform as rio_transform
-import simplejson as json
 import xarray as xr
 from shapely import ops
 from shapely.geometry import LineString, Point, Polygon, box
@@ -252,6 +253,7 @@ def gtiff2xarray(
     r_dict: Dict[str, bytes],
     geometry: Union[Polygon, Tuple[float, float, float, float]],
     geo_crs: str,
+    ds_dims: Tuple[str, str] = ("y", "x"),
 ) -> Union[xr.DataArray, xr.Dataset]:
     """Convert responses from ``pygeoogc.wms_bybox`` to ``xarray.Dataset``.
 
@@ -263,40 +265,52 @@ def gtiff2xarray(
         The geometry to mask the data that should be in the same CRS as the r_dict.
     geo_crs : str
         The spatial reference of the input geometry.
+    ds_dims : tuple of str, optional
+        The names of the vertical and horizontal dimensions (in that order)
+        of the target dataset, default to ("y", "x").
 
     Returns
     -------
     xarray.Dataset or xarray.DataAraay
         The dataset or data array based on the number of variables.
     """
-    var_name = {
-        lyr: "_".join(lyr.split("_")[:-2]) if "_dd_" in lyr else lyr for lyr in r_dict.keys()
-    }
+    key1 = next(iter(r_dict.keys()))
+    if len(r_dict) == 1 and "dd" not in key1:
+        r_dict = {f"{key1}_dd_0_0": r_dict[key1]}
+
+    var_name = {lyr: "_".join(lyr.split("_")[:-3]) for lyr in r_dict.keys()}
 
-    attrs = {}
     with rio.MemoryFile() as memfile:
-        memfile.write(list(r_dict.values())[0])
+        memfile.write(r_dict[next(iter(r_dict.keys()))])
         with memfile.open() as src:
-            attrs["transform"] = src.transform
-            attrs["res"] = (src.transform[0], src.transform[4])
-            attrs["bounds"] = tuple(src.bounds)
-            attrs["nodatavals"] = src.nodatavals
-            attrs["crs"] = src.crs
+            r_crs = pyproj.CRS.from_user_input(src.crs)
+            if src.nodata is None:
+                try:
+                    nodata = np.iinfo(src.dtypes[0]).max
+                except ValueError:
+                    nodata = np.nan
+            else:
+                nodata = np.dtype(src.dtypes[0]).type(src.nodata)
 
-    _geometry = [geo2polygon(geometry, geo_crs, attrs["crs"])]
+    _geometry = geo2polygon(geometry, geo_crs, r_crs)
 
-    def _create_dataset(content: bytes, name: str) -> Union[xr.Dataset, xr.DataArray]:
+    def to_dataset(lyr: str) -> xr.DataArray:
         with rio.MemoryFile() as memfile:
-            memfile.write(content)
+            memfile.write(r_dict[lyr])
             with memfile.open() as src:
-                out_image, out_transform = rio_mask.mask(src, _geometry, crop=True)
+                geom = [_geometry.intersection(box(*src.bounds))]
+                if geom[0].is_empty:
+                    msk, transform, _ = rio_mask.raster_geometry_mask(src, [_geometry], invert=True)
+                else:
+                    msk, transform, _ = rio_mask.raster_geometry_mask(src, geom, invert=True)
                 meta = src.meta
                 meta.update(
                     {
                         "driver": "GTiff",
-                        "height": out_image.shape[1],
-                        "width": out_image.shape[2],
-                        "transform": out_transform,
+                        "height": msk.shape[0],
+                        "width": msk.shape[1],
+                        "transform": transform,
+                        "nodata": nodata,
                     }
                 )
 
@@ -306,16 +320,16 @@ def gtiff2xarray(
                         ds = ds.squeeze("band", drop=True)
                     except ValueError:
                         pass
-                    ds.name = name
-        return ds
+                    coords = {ds_dims[0]: ds.coords[ds_dims[0]], ds_dims[1]: ds.coords[ds_dims[1]]}
+                    msk_da = xr.DataArray(msk, coords, dims=ds_dims)
+                    ds = ds.where(msk_da, drop=True)
+                    ds.attrs["crs"] = r_crs
+                    ds.name = var_name[lyr]
+                    return ds
 
-    ds = xr.merge([_create_dataset(r, var_name[lyr]) for lyr, r in r_dict.items()])
-
-    for a, v in attrs.items():
-        if a not in ds.attrs:
-            ds.attrs[a] = v
-
-    ds = xarray_geomask(ds, _geometry[0], attrs["crs"])
+    ds = xr.merge(to_dataset(lyr) for lyr in r_dict.keys())
+    ds.attrs["crs"] = r_crs
+    ds.attrs["transform"] = get_transform(_geometry, ds.sizes[ds_dims[0]], ds.sizes[ds_dims[1]])
 
     if len(ds.variables) - len(ds.dims) == 1:
         ds = ds[list(ds.keys())[0]]
@@ -326,6 +340,7 @@ def xarray_geomask(
     ds: Union[xr.Dataset, xr.DataArray],
     geometry: Union[Polygon, Tuple[float, float, float, float]],
     geo_crs: str,
+    ds_dims: Tuple[str, str] = ("y", "x"),
 ):
     """Mask a ``xarray.Dataset`` based on a geometry.
 
@@ -337,20 +352,64 @@ def xarray_geomask(
         The geometry or bounding box to mask the data
     geo_crs : str
         The spatial reference of the input geometry
+    ds_dims : tuple of str, optional
+        The names of the vertical and horizontal dimensions (in that order)
+        of the dataset, default to ("y", "x").
 
     Returns
     -------
     xarray.Dataset or xarray.DataArray
         The input dataset with a mask applied (np.nan)
     """
-    _geometry = geo2polygon(geometry, geo_crs, ds.crs)
-    west, south, east, north = _geometry.bounds
-    height, width = ds.sizes["y"], ds.sizes["x"]
-    transform = rio_transform.from_bounds(west, south, east, north, width, height)
-    _mask = rio_features.geometry_mask([_geometry], (height, width), transform, invert=True)
-    mask = xr.DataArray(_mask, dims=("y", "x"))
+    if not isinstance(ds_dims, tuple) or len(ds_dims) != 2:
+        raise InvalidInputType("ds_dims", "tuple of length 2", '("y", "x")')
 
-    return ds.where(mask, drop=True)
+    if "crs" not in ds.attrs:
+        raise ValueError("The input dataset is missing the crs attribute.")
+
+    _geometry = geo2polygon(geometry, geo_crs, ds.crs)
+    height, width = ds.sizes[ds_dims[0]], ds.sizes[ds_dims[1]]
+    transform = get_transform(_geometry, height, width)
+
+    _mask = rio_features.geometry_mask([_geometry], (height, width), transform, invert=True)
+
+    coords = {ds_dims[0]: ds.coords[ds_dims[0]], ds_dims[1]: ds.coords[ds_dims[1]]}
+    mask = xr.DataArray(_mask, coords, dims=ds_dims)
+
+    ds_masked = ds.where(mask, drop=True)
+    ds_masked.attrs["transform"] = transform
+    ds_masked.attrs["bounds"] = _geometry.bounds
+
+    return ds_masked
+
+
+def get_transform(
+    geometry: Union[Polygon, Tuple[float, float, float, float]], width: int, height: int
+) -> affine.Affine:
+    """Get transform of a Polygon or bounding box.
+
+    Parameters
+    ----------
+    geometry : Polygon or tuple of float
+        The geometry or the bounding box, (west, south, east, north).
+    width: int
+        The width of the target raster in pixels.
+    height: int
+        The height of the target raster in pixels.
+
+    Returns
+    -------
+    affin.Affine
+        The affine transform of the geometry.
+    """
+    if isinstance(geometry, Polygon):
+        west, south, east, north = geometry.bounds
+    elif isinstance(geometry, tuple) and len(geometry) == 4:
+        west, south, east, north = geometry
+    else:
+        raise InvalidInputType("geometry", "Polygon or (west, south, east, north)")
+
+    return rio_transform.from_bounds(west, south, east, north, width, height)
 
 
 class MatchCRS:
